#include <Redis_Provider.h>
#include <Base_Configurator.h>

using namespace inaf::oasbo::Providers;
RedisProvider::RedisProvider() {
	RedisProvider("127.0.0.1", 6379, "DAQ_key");
}

RedisProvider::RedisProvider(std::string ip, int port, std::string key) {
	setDest(key);
	this->port = port;
	this->ip = ip;
}

int RedisProvider::write(PacketLib::BasePacket &packet) {
	return write(packet, this->dest);
}

int RedisProvider::write(PacketLib::BasePacket &packet, std::string key) {
	if (!isOpen()) {
		if (context) {
			std::cout << "Redis Provider Error: " << context->errstr
					<< std::endl;
		} else {
			std::cout << "Redis Provider Error: Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}
	uint size = packet.getHeaderSize() + packet.getPayloadSize()
			+ packet.getTailSize();
	redisReply *r = (redisReply*) redisCommand(context, "LPUSH %s %b",
			key.c_str(), (char*) packet.getBinaryPointer(), size);
	if (r == NULL) {
		// Error executing Redis command
		return -1;
	}
	// Check the type of the Redis reply
	if (r->type == REDIS_REPLY_ERROR) {
		// Error executing Redis command
		std::cout << r->str << std::endl;
		freeReplyObject(r);
		return -1;
	}
	freeReplyObject(r);
	return size;
}

int RedisProvider::open() {
	if (isOpen())
		close();
	context = redisConnect(ip.c_str(), port);
	if (context == nullptr || context->err) {
		if (context) {
			std::cout << "Redis Provider Error: " << context->errstr
					<< std::endl;
		} else {
			std::cout << "Redis Provider Error: Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}
	return 1;
}

int RedisProvider::close() {
	if (context != nullptr && !context->err) {
		redisFree(this->context);
		context = nullptr;
	}
	return 1;
}

bool RedisProvider::isOpen() {
	return (context != nullptr && !context->err);
}

RedisProvider::~RedisProvider() {
	close();
}

using R_B = RedisProvider::Builder;
R_B::Builder() {
	this->reset();
}
R_B::Builder(std::string ip, int port, std::string topic) {
	this->provider = new RedisProvider(ip, port, topic);

}
R_B::~Builder() {
	delete provider;
}

void R_B::reset() {
	this->provider = new RedisProvider();
}

R_B* R_B::configFrom(Configurators::BaseConfigurator *conf) {
	std::map<std::string, std::string> params = conf->readConfig();
	if (params.count(ip_key) > 0)
		provider->ip = params[ip_key];
	if (params.count(port_key) > 0)
		provider->port = std::stoi(params[port_key]);
	if (params.count(redis_key_key) > 0)
		provider->setDest(params[redis_key_key]);
	return this;
}

R_B* R_B::setIp(std::string ip) {
	provider->ip = ip;
	return this;
}

R_B* R_B::setPort(int port) {
	provider->port = port;
	return this;
}

R_B* R_B::setKey(std::string key) {
	provider->setDest(key);
	return this;
}

RedisProvider* R_B::getProvider() {
	RedisProvider *result = this->provider;
	this->reset();
	return result;
}
